@@ -56,8 +56,8 @@ import logging
 
 # websocket modules
 from ._exceptions import *
-from ._abnf import ABNF
-from ._utils import NoLock
+from ._abnf import *
+from ._utils import NoLock, validate_utf8
 
 """
 websocket python client.
@@ -71,19 +71,6 @@ Please see http://tools.ietf.org/html/rfc6455 for protocol.
 # websocket supported version.
 VERSION = 13
 
-# closing frame status codes.
-STATUS_NORMAL = 1000
-STATUS_GOING_AWAY = 1001
-STATUS_PROTOCOL_ERROR = 1002
-STATUS_UNSUPPORTED_DATA_TYPE = 1003
-STATUS_STATUS_NOT_AVAILABLE = 1005
-STATUS_ABNORMAL_CLOSED = 1006
-STATUS_INVALID_PAYLOAD = 1007
-STATUS_POLICY_VIOLATION = 1008
-STATUS_MESSAGE_TOO_BIG = 1009
-STATUS_INVALID_EXTENSION = 1010
-STATUS_UNEXPECTED_CONDITION = 1011
-STATUS_TLS_HANDSHAKE_ERROR = 1015
 
 DEFAULT_SOCKET_OPTION = [(socket.SOL_TCP, socket.TCP_NODELAY, 1),]
 if hasattr(socket, "SO_KEEPALIVE"):
@@ -208,6 +195,7 @@ def create_connection(url, timeout=None, **options):
              "enable_multithread" -> enable lock for multithread.
              "sockopt" -> socket options
              "sslopt" -> ssl option
+             "subprotocols" - array of available sub protocols. default is None.
     """
     sockopt = options.get("sockopt", [])
     sslopt = options.get("sslopt", {})
@@ -358,6 +346,8 @@ class WebSocket(object):
         else:
             self.lock = NoLock()
 
+        self.subprotocol = None
+
     def fileno(self):
         return self.sock.fileno()
 
@@ -410,6 +400,7 @@ class WebSocket(object):
                  "cookie" -> cookie value.
                  "http_proxy_host" - http proxy host name.
                  "http_proxy_port" - http proxy port. If not set, set to 80.
+                 "subprotocols" - array of available sub protocols. default is None.
 
         """
 
@@ -509,6 +500,10 @@ class WebSocket(object):
         headers.append("Sec-WebSocket-Key: %s" % key)
         headers.append("Sec-WebSocket-Version: %s" % VERSION)
 
+        subprotocols = options.get("subprotocols")
+        if subprotocols:
+            headers.append("Sec-WebSocket-Protocol: %s" % ",".join(subprotocols))
+
         if "header" in options:
             headers.extend(options["header"])
 
@@ -530,14 +525,14 @@ class WebSocket(object):
         _dump("request header", header_str)
 
         resp_headers = self._get_resp_headers()
-        success = self._validate_header(resp_headers, key)
+        success = self._validate_header(resp_headers, key, options.get("subprotocols"))
         if not success:
             self.close()
             raise WebSocketException("Invalid WebSocket Header")
 
         self.connected = True
 
-    def _validate_header(self, headers, key):
+    def _validate_header(self, headers, key, subprotocols):
         for k, v in _HEADERS_TO_CHECK.items():
             r = headers.get(k, None)
             if not r:
@@ -545,6 +540,14 @@ class WebSocket(object):
             r = r.lower()
             if v != r:
                 return False
+        
+        if subprotocols:
+            subproto = headers.get("sec-websocket-protocol", None)
+            if not subproto or subproto not in subprotocols:
+                logger.error("Invalid subprotocol: " + str(subprotocols))
+                return False
+            self.subprotocol = subproto
+
 
         result = headers.get("sec-websocket-accept", None)
         if not result:
@@ -678,41 +681,8 @@ class WebSocket(object):
 
         return  value: tuple of operation code and string(byte array) value.
         """
-        while True:
-            frame = self.recv_frame()
-            if not frame:
-                # handle error:
-                # 'NoneType' object has no attribute 'opcode'
-                raise WebSocketException("Not a valid frame %s" % frame)
-            elif frame.opcode in (ABNF.OPCODE_TEXT, ABNF.OPCODE_BINARY, ABNF.OPCODE_CONT):
-                if frame.opcode == ABNF.OPCODE_CONT and not self._recving_frames:
-                    raise WebSocketException("Illegal frame")
-
-                if self._cont_data:
-                    self._cont_data[1] += frame.data
-                else:
-                    if frame.opcode in (ABNF.OPCODE_TEXT, ABNF.OPCODE_BINARY):
-                        self._recving_frames = frame.opcode
-                    self._cont_data = [frame.opcode, frame.data]
-
-                if frame.fin:
-                    self._recving_frames = None
-                
-                if frame.fin or self.fire_cont_frame:
-                    data = self._cont_data
-                    self._cont_data = None
-                    return data
-
-            elif frame.opcode == ABNF.OPCODE_CLOSE:
-                self.send_close()
-                return (frame.opcode, frame.data)
-            elif frame.opcode == ABNF.OPCODE_PING:
-                self.pong(frame.data)
-                if control_frame:
-                    return (frame.opcode, frame.data)
-            elif frame.opcode == ABNF.OPCODE_PONG:
-                if control_frame:
-                    return (frame.opcode, frame.data)
+        opcode, frame = self.recv_data_frame(control_frame)
+        return opcode, frame.data
 
     def recv_data_frame(self, control_frame=False):
         """
@@ -728,24 +698,39 @@ class WebSocket(object):
             if not frame:
                 # handle error:
                 # 'NoneType' object has no attribute 'opcode'
-                raise WebSocketException("Not a valid frame %s" % frame)
+                raise WebSocketProtocolException("Not a valid frame %s" % frame)
             elif frame.opcode in (ABNF.OPCODE_TEXT, ABNF.OPCODE_BINARY, ABNF.OPCODE_CONT):
-                if frame.opcode == ABNF.OPCODE_CONT and not self._cont_data:
-                    raise WebSocketException("Illegal frame")
-                if self._cont_data:
-                    self._cont_data[1].data += frame.data
-                else:
-                    self._cont_data = [frame.opcode, frame]
+                if not self._recving_frames and frame.opcode == ABNF.OPCODE_CONT:
+                    raise WebSocketProtocolException("Illegal frame")
+                if self._recving_frames and frame.opcode in (ABNF.OPCODE_TEXT, ABNF.OPCODE_BINARY):
+                    raise WebSocketProtocolException("Illegal frame")
 
+                if self._cont_data:
+                    self._cont_data[1] += frame.data
+                else:
+                    if frame.opcode in (ABNF.OPCODE_TEXT, ABNF.OPCODE_BINARY):
+                        self._recving_frames = frame.opcode
+                    self._cont_data = [frame.opcode, frame.data]
+
+                if frame.fin:
+                    self._recving_frames = None
+                
                 if frame.fin or self.fire_cont_frame:
                     data = self._cont_data
                     self._cont_data = None
-                    return data
+                    frame.data = data[1]
+                    if not self.fire_cont_frame and data[0] == ABNF.OPCODE_TEXT and not validate_utf8(frame.data):
+                        raise WebSocketPayloadException("cannot decode: " + repr(frame.data))
+                    return [data[0], frame]
+
             elif frame.opcode == ABNF.OPCODE_CLOSE:
                 self.send_close()
                 return (frame.opcode, frame)
             elif frame.opcode == ABNF.OPCODE_PING:
-                self.pong(frame.data)
+                if len(frame.data) < 126:
+                    self.pong(frame.data)
+                else:
+                    raise WebSocketProtocolException("Ping message is too long")
                 if control_frame:
                     return (frame.opcode, frame)
             elif frame.opcode == ABNF.OPCODE_PONG:
@@ -782,7 +767,10 @@ class WebSocket(object):
         # Reset for next frame
         frame_buffer.clear()
 
-        return ABNF(fin, rsv1, rsv2, rsv3, opcode, has_mask, payload)
+        frame = ABNF(fin, rsv1, rsv2, rsv3, opcode, has_mask, payload)
+        frame.validate()
+
+        return frame
 
 
     def send_close(self, status=STATUS_NORMAL, reason=six.b("")):
@@ -828,15 +816,21 @@ class WebSocket(object):
             except:
                 pass
 
-        self._closeInternal()
+        self.shutdown()
 
-    def _closeInternal(self):
-        self.sock.close()
+    def shutdown(self):
+        "close socket, immediately."
+        if self.sock:
+            self.sock.close()
+            self.sock = None
 
     def _send(self, data):
         if isinstance(data, six.text_type):
             data = data.encode('utf-8')
 
+        if not self.sock:
+            raise WebSocketConnectionClosedException("socket is already closed.")
+
         try:
             return self.sock.send(data)
         except socket.timeout as e:
@@ -850,6 +844,9 @@ class WebSocket(object):
                 raise
 
     def _recv(self, bufsize):
+        if not self.sock:
+            raise WebSocketConnectionClosedException("socket is already closed.")
+
         try:
             bytes = self.sock.recv(bufsize)
         except socket.timeout as e:
