@@ -25,14 +25,14 @@ import uuid
 # websocket-client
 import websocket as ws
 from websocket._core import _parse_url, _create_sec_websocket_key
+from websocket._utils import validate_utf8
 
 
 # Skip test to access the internet.
-TEST_WITH_INTERNET = True
+TEST_WITH_INTERNET = False
 
 # Skip Secure WebSocket test.
 TEST_SECURE_WS = False
-
 TRACABLE = False
 
 
@@ -184,25 +184,31 @@ class WebSocketTest(unittest.TestCase):
             "connection": "upgrade",
             "sec-websocket-accept": "Kxep+hNu9n51529fGidYu7a3wO0=",
             }
-        self.assertEqual(sock._validate_header(required_header, key), True)
+        self.assertEqual(sock._validate_header(required_header, key, None), True)
 
         header = required_header.copy()
         header["upgrade"] = "http"
-        self.assertEqual(sock._validate_header(header, key), False)
+        self.assertEqual(sock._validate_header(header, key, None), False)
         del header["upgrade"]
-        self.assertEqual(sock._validate_header(header, key), False)
+        self.assertEqual(sock._validate_header(header, key, None), False)
 
         header = required_header.copy()
         header["connection"] = "something"
-        self.assertEqual(sock._validate_header(header, key), False)
+        self.assertEqual(sock._validate_header(header, key, None), False)
         del header["connection"]
-        self.assertEqual(sock._validate_header(header, key), False)
+        self.assertEqual(sock._validate_header(header, key, None), False)
 
         header = required_header.copy()
         header["sec-websocket-accept"] = "something"
-        self.assertEqual(sock._validate_header(header, key), False)
+        self.assertEqual(sock._validate_header(header, key, None), False)
         del header["sec-websocket-accept"]
-        self.assertEqual(sock._validate_header(header, key), False)
+        self.assertEqual(sock._validate_header(header, key, None), False)
+
+
+        header = required_header.copy()
+        header["sec-websocket-protocol"] = "sub1"
+        self.assertEqual(sock._validate_header(header, key, ["sub1", "sub2"]), True)
+        self.assertEqual(sock._validate_header(header, key, ["sub2", "sub3"]), False)
 
     def testReadHeader(self):
         sock = ws.WebSocket()
@@ -424,12 +430,11 @@ class WebSocketTest(unittest.TestCase):
 
     @unittest.skipUnless(TEST_WITH_INTERNET, "Internet-requiring tests are disabled")
     def testAfterClose(self):
-        from socket import error
         s = ws.create_connection("ws://echo.websocket.org/")
         self.assertNotEqual(s, None)
         s.close()
-        self.assertRaises(error, s.send, "Hello")
-        self.assertRaises(error, s.recv)
+        self.assertRaises(ws.WebSocketConnectionClosedException, s.send, "Hello")
+        self.assertRaises(ws.WebSocketConnectionClosedException, s.recv)
 
     def testUUID4(self):
         """ WebSocket key should be a UUID4.
@@ -519,6 +524,14 @@ class SockOptTest(unittest.TestCase):
         self.assertNotEqual(s.sock.getsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY), 0)
         s.close()
 
+class UtilsTest(unittest.TestCase):
+    def testUtf8Validator(self):
+        state = validate_utf8(six.b('\xf0\x90\x80\x80'))
+        self.assertEqual(state, True) 
+        state = validate_utf8(six.b('\xce\xba\xe1\xbd\xb9\xcf\x83\xce\xbc\xce\xb5\xed\xa0\x80edited'))
+        self.assertEqual(state, False)
+        state = validate_utf8(six.b(''))
+        self.assertEqual(state, True)
 
 if __name__ == "__main__":
     unittest.main()
